rm(list=ls(all=TRUE))
library(mvabund)
source("te_test_functions.r")
# Abundance data (75 sites x 82 species)


# Case study: aravo data set of Choler 2005 in ade4 -----------------------

library(ade4)
data(aravo)
Y <-      aravo$spe
Traits <- aravo$traits
Environment <-  aravo$env

# 
# # Case study: aravo data set of Choler 2005 from csv files ----------------
# Y <- read.csv("aravo_Y.csv")
# Traits <- read.csv("aravo_traits.csv")
# Environment <- read.csv("aravo_environment.csv")
#

# log transform of Spread
Traits$Spread = log(Traits$Spread)
#

summary(Environment)
summary(Traits)
round(cor(Traits[,c("Spread","SLA")]),2)


# Check on mean-variance relation via residual plot (Fig S1.pdf of the paper) ----------


obj <- make_obj_for_traitenv(Y, E=Environment[,"Snow"], T = matrix(1,nrow= nrow(Traits),ncol=1))
XYZ <- expand4glm(obj) # ET1 is the product of Snow and trait which is a ones vector in this case
names(XYZ)[4]= "Snow"  # rename ET1 to Snow
ftE <- manyglm(y~site+species+ species:Snow, data = XYZ, family= "negative.binomial", theta.method = "Chi2")
plot(ftE)


#  Spread-Snow association ------------------------------

nrepet <- 39 # was 999 in the paper
obj <- make_obj_for_traitenv(Y, E=Environment[,"Snow"], T = Traits[,"Spread"])
L = obj$L; E = obj$E; T = obj$T


# anova.traitglm test (site-based bootstrap using negative binomial likelihood) -----------------------------------
# WARNING takes long to run
mod0 <- traitglm(L,E,T, formula = ~1, family= "negative.binomial", method="manyglm", composition="TRUE")
mod <- traitglm(L,E,T, family= "negative.binomial", method="manyglm", composition="TRUE")
an_Spread_Snow <- anova.traitglm(mod0, mod, nBoot=nrepet)

# sites, species, max r/c tests (site- / species- / max r/c based permutation using poisson likelihood -----------------------------------

res_glm_pmax <- pmax_PermutationTest(obj, trait_env_glm_LR, family_ = poisson, nrepet = nrepet)
res_glm_pmax

# sites, species, max r/c tests (site- / species- / max r/c based permutation using squared fourth corner instead of Poisson likelihood -----------------------------------

res_4th_pmax <- pmax_PermutationTest(obj, fourthcorner, nrepet = nrepet, withglm = FALSE)
res_4th_pmax

#  sites test using ade4:fourthcorner -----------------------------------
library(ade4)
fourth_ade4_sites <- ade4::fourthcorner(as.data.frame(obj$E),as.data.frame(obj$L),as.data.frame(obj$T), modeltype = 2, nrepet = nrepet)
fourth_ade4_sites

#  species test using ade4:fourthcorner -----------------------------------
fourth_ade4_species <- ade4::fourthcorner(as.data.frame(obj$E),as.data.frame(obj$L),as.data.frame(obj$T), modeltype = 4, nrepet = nrepet)
fourth_ade4_species

#  max r/c test using ade4:fourthcorner -----------------------------------
#  default modeltype = 6 (this is the max r/c test using squared fourthcorner correlation)
fourth_ade4_max_rc <- ade4::fourthcorner(as.data.frame(obj$E),as.data.frame(obj$L),as.data.frame(obj$T), nrepet = nrepet)
fourth_ade4_max_rc

#  predictive modelling using lasso with site-based cross-validation -----------------------------------

ft1 <- traitglm(L,E,T, family= "negative.binomial", method = "cv.glm1path", plot=TRUE, composition=TRUE)
ft1$coefficients
print(ft1$fourth.corner)
# try also with composition is FALSE
ft2 <- traitglm(L,E,T, family= "negative.binomial", method = "cv.glm1path", plot=TRUE, composition=FALSE)
ft2$coefficients
print(ft2$fourth.corner)



# predictive modelling using lasso with species-based cross-validation -----------------------------------

ft1Transpose <- traitglm(t(L),T,E, family= "negative.binomial", method = "cv.glm1path", plot=TRUE, composition=TRUE)
ft1Transpose$coefficients
print(ft1Transpose$fourth.corner)

ft2Transpose <- traitglm(t(L),T,E, family= "negative.binomial", method = "cv.glm1path", plot=TRUE, composition=FALSE)
ft2Transpose$coefficients
print(ft2Transpose$fourth.corner)








