# make object from data (L=Y) with checks on empty row/cols ---------------------------------

make_obj_for_traitenv <- function(L,E1,T1, cut_off = 0){
  # makes object of class TE_obj from matrices or dataframes from L,E1 and T1
  # no factors allowed, as all is converted to matrices
  # the object is a list of three matrices: L, E, T
  L <- as.matrix(L)
  E1 <- as.matrix(E1)
  T1 <- as.matrix(T1)
  # check_L()
  rows<-seq_len(nrow(L))
  cols<-seq_len(ncol(L))
  rni <-which(rowSums(L)==0)
  repeat {
    if (length(rni)) {L <- L[-rni,,drop = FALSE]; rows <-rows[-rni]}
    ksi <- which(colSums(L)==0)
    if (length(ksi)) {L <- L[,-ksi, drop = FALSE]; cols <- cols[-ksi]}
    rni <-which(rowSums(L)==0)
    if ( length(rni)==0 & length(ksi)==0){break}
  }
  E1 <-as.matrix(E1)[rows,,drop = FALSE]
  T1 <-as.matrix(T1)[cols,,drop = FALSE]
  # end check_L()
  if(cut_off >0 ){
    abOcc <- apply(L>0,2,mean)
    L <- L[,abOcc>cut_off, drop =FALSE]
    T1 <- as.matrix(T1)[abOcc>cut_off,, drop = FALSE]
  } 
  if(is.null(rownames(L)))rownames(L)<- paste("site", 1:nrow(L), sep="")
  if(is.null(colnames(L)))colnames(L)<- paste("spec",1:ncol(L),sep="")
  rownames(E1)= rownames(L); colnames(E1)=  paste("E", 1:ncol(E1), sep = "")
  rownames(T1)= colnames(L); colnames(T1)=  paste("T", 1:ncol(T1), sep = "") 
  obj = list(L=L, E=E1,T = T1)
  class(obj) = c("TE_obj", class(obj))
  return(obj)
}


# convert lists of data frames to matrices and the reverse ----------------

df2matrix <- function(obj){
  # makes matrices from the data frames in obj 
  lapply(obj,as.matrix)
}

matrix2df <- function(obj){
  # makes dataframes from the matrices in obj 
   lapply(obj,as.data.frame)
}


# function doing traitglm and anova.traitglm on obj -----------------------

anova_traitglm <- function(obj, family = "negative.binomial", composition=TRUE, nrepet = 39){
  # obj is from make_obj_for_traitenv or its dataframe version
  # nrepet is number of bootstraps in anova.traitglm
  # value:  c("Devdiff","pval")
  if (class(obj)[1]=="TE_obj") obj <- matrix2df(obj)
  with(obj,{
    ft0 <- try(traitglm(L,E,T,formula = ~1, family= family, method="manyglm", composition=composition))
    # ft1 <- try(traitglm(L,E,T,formula = ~E1:T1, family= family, method="manyglm", composition=composition))
    # use all interactions: no formula!
    # we need  try() here as sometimes there is trouble. Even this does not work aways.
    ft1 <- try(traitglm(L,E,T, family= family, method="manyglm", composition=composition))
    if (class(ft0)[1]=="traitglm" && class(ft1)[1]=="traitglm"){
      Devdiff <- ft1$two.loglike - ft0$two.loglike #== Dev in anova.manyglm(ft0,ft1,nBoot=1)
      an <- try(anova.traitglm(ft0,ft1,nBoot=nrepet, show.time = "none")  )    
      if (class(an)=="anova.manyglm") pval  <- an$table$"Pr(>Dev)"[2] else pval <- NA
      result <- c(Devdiff, pval)
    } else result <- c(NA,NA) 
    names(result)= c("Devdiff","pval")
    return(result)
  })
}

# special purpose functions for glm replacing traitglm ----------------------------------

# 1. expand the data in the obj (L,E,T) in vector form -----------------------
expand4glm <- function(obj){
  if (class(obj)[1]=="TE_obj") obj <- matrix2df(obj)
  # adapted from Jamil et al 2013; one environmental variable only, 1-many traits!
  with(obj, {
    sitespec <- expand.grid(rownames(L),colnames(L))
    site <-sitespec[,1]; species<-sitespec[,2]
    y <- as.vector(as.matrix(L))
    Evec <-  E[site,, drop = FALSE];  rownames(Evec)= NULL
    Tvec <-  T[species,, drop = FALSE]; rownames(Tvec)= NULL
    # for one Evec!!!
    ET <- Tvec * Evec[,1]; colnames(ET)= paste("E", colnames(T),sep="")
    XYZ <- data.frame(y,site,species,ET)
    return(XYZ) # XYZ notation of Jamil et al 2013
  })
}

trait_env_glm_LR <- function(obj, family_ = poisson, composition=TRUE, fitH0){
  # trait_env_glm_LR calculates the deviance difference (=2log(LR)) between 
  # the null (R+C) model and the R+C+et model for resampling
  # the R+C null model does not depend on E and T and thus remains constant during resampling
  # used via fitH0
  # 
  # obj: objectfrom make_obj_for_traitenv for all preprocessing
  # fitH0: list(deviance, eta, formula_with_ET) with deviance and eta (linear predictor) of null model
  #           and the formula for the alternative model (fitH0$formula_with_ET)
  # value: the deviance differene as used in anova.manyglm
  XYZ <- expand4glm(obj)
  fit_with_ET <- glm(fitH0$formula_with_ET, family = family_, data = XYZ, etastart = fitH0$eta)
  Devdiff <-  fitH0$deviance - fit_with_ET$deviance
  return(c(glmRCmax=Devdiff))
}

pmax_PermutationTest <- function(obj, FUN_test_statistics=trait_env_glm_LR, nrepet = 39, withglm=TRUE, ...) {
  # obj from make_obj_for_traitenv(L,E,T, cut_off): matrices L,E and T (one environmental variable, 1-many traits)
  # FUN_test_statistic : function that returns one or more test statistics
  # e.g.FUN_test_statistic = trait_env__glm_LR or test_statistics_TE; 
  # ... options for FUN_test_statistic
  # NOTE: the reuse of the null model (fitH0 below) is only valid with compositional is true, or without any row main effects/ any column main effects
  # function uses: composition == TRUE
  
  if  (withglm){
    XYZ <- expand4glm(obj) # data frame XYZ
    #names(XYZ)
    gl0 <- glm(y~site+species, family = poisson, data = XYZ)
    formula_with_ET <- as.formula(paste( "y~site+species+",paste(paste("ET", 1 : ncol(obj$T),sep =""),collapse = "+"),sep=""))
    fitH0 <- list(deviance= gl0$deviance, eta = gl0$linear.predictors, formula_with_ET=formula_with_ET)
  } else fitH0 = NA
  obs <- FUN_test_statistics(obj,fitH0=fitH0, ...)
  sim.row <- matrix(0, nrow = nrepet, ncol = length(obs))
  sim.col <- matrix(0, nrow = nrepet, ncol = length(obs))
  #sim.rc <- matrix(0, nrow = nrepet, ncol = length(obs))
  for(i in 1:nrepet){
    per.row <- sample(nrow(obj$L))
    per.col <- sample(ncol(obj$L))
    #permute_rows_columns <- function(obj, per.row,per.col){
    obj.row <- obj.col <-  obj  # obj.rc <- obj
    obj.row$E <-obj.row$E[per.row,,drop =FALSE]
    obj.col$T <-obj.col$T[per.col,,drop =FALSE]
    sim.row[i, ] <- FUN_test_statistics(obj.row,fitH0=fitH0, ...)
    sim.col[i, ] <- FUN_test_statistics(obj.col,fitH0=fitH0, ...)
  }
  if (length(obs)==1){
    pval.row <- (sum(abs(sim.row) >= abs(obs), na.rm = TRUE) + 1)  / (nrepet + 1)
    pval.col <- (sum(abs(sim.col) >= abs(obs), na.rm = TRUE) + 1)  / (nrepet + 1)
    
  } else {
    obs.mat <- matrix(rep(abs(obs),each=nrepet), nrow= nrepet, ncol=length(obs))
    isna <-  colSums(is.na(sim.row))
    pval.row <- (colSums(abs(sim.row) >= obs.mat, na.rm=TRUE) + 1)/ (nrepet - isna + 1)
    isna <-  colSums(is.na(sim.col))
    pval.col <- (colSums(abs(sim.col) >= obs.mat, na.rm=TRUE) + 1)/ (nrepet - isna + 1)
  }
  result <- cbind(test_stat = obs, prow = pval.row, pcol = pval.col, pmax = pmax(pval.row, pval.col))
  return(result)
}



# # function using parametric glmm (similar to anova_traitglm, but no sampling)----------------
# library(lme4)
# fourthcornerglmm0 <- function(obj, family.glmm=poisson(link = "log")){
# # determine the parametric test on interaction from the glmm of Jamil et al 2013 for count data
# #  obj <- make_obj_for_traitenv(L,E,T, cut_off)
# #  obj <- expand4glmm(obj)
#   if (is.matrix(obj$L)) obj <- matrix2df(obj)
#   if (length(obj)==3) obj <- expand4glmm(obj) 
#   fm0 = try(glmer(y ~ site + species + (0 + E1 | species)+ (1|site.spec),
#                 family=family.glmm,data=obj))
#   fm1 = try(update(fm0, formula= . ~. + E1:T1 ))
#   if( class(fm0)=="glmerMod" && class(fm1)=="glmerMod" ){
#     an <- anova(fm1,fm0)
#     pval <- an$"Pr(>Chisq)"[2]
#     Devdiff <-an$deviance[1]- an$deviance[2]
#   } else { pval <- NA; Devdiff <- NA}
#   result <- c(Devdiff=Devdiff, pval=pval)
#   names(result)= c("Devdiff","pval")
#   return(result)
# }

# fourth corner test statistic (could use ade4 package instead... )--------------------------------------------
fourthcorner <- function(obj,...){
  # fourthcorner function for resampling for numeric data
  # obj is from make_obj_for_traitenv for all preprocessing
  # ... added to allow fourthcorner as FUN_test_statistics in pmax_PermutationTest
  # value: the sum of the squares of the fourthcorner correlation(s) between E and T (can be matrices)
  #
  # local functions
  center_w <- function(X,w = rep(1/nrow(X),nrow(X))){ X - rep(1,length(w))%*%t(w)%*% X }
  standardize_w <- function(X,w = rep(1/nrow(X),nrow(X))){
    ones <- rep(1,length(w))
    Xc <- X - ones %*% t(w)%*% X
    Xc / ones%*%sqrt(t(ones)%*%(Xc*Xc*w)) 
  }
  # end of local functions
  #if(is.data.frame(obj$L)) obj <- df2matrix(obj)
  with(obj,{
    L<-L/sum(L)
    Wn <- rowSums(L)
    Ws <- colSums(L)
    Estd_w <- standardize_w(E,Wn)
    Tstd_w <-  standardize_w(T,Ws)
    # Fourth corner calculated as W_n weighted covariance between 
    Fourthcorner <- t(Estd_w)%*%(L%*%Tstd_w)
    Fourthcorner <- sum(Fourthcorner*Fourthcorner)
    return(Fourthcorner=Fourthcorner)
  })
}


test_statistics_TE <- function(obj, family=poisson,  fitH0){
  # function to generate test statistics from an obj from make_obj_for_traitenv (TE_obj object)
  # for use in RC_pmax_PermutationTest 
  # value: a named vector of all test statistics 
  fourth <- fourthcorner(obj) 
  trglm <- trait_env_glm_LR(obj, family, composition=TRUE, fitH0=fitH0)
  res <- c(trglm, fourth) 
  names(res)= c("glm","fourth")
  return(res)
}

