rm(list=ls(all=TRUE))  # remove all existing items from the workspace
library(mvabund)
source("generate_trait_env_community_data.r")
source("te_test_functions.r")


# Section 1a Gaussian response simulations with 10 random traits --------------------------------------------------
# n = 20; m = 11; q = 10 
n_communities <- 20 #n
n_species <- 11 # m
mu_max <- 10
n_traits <- 10  #q
cor_Tdat <- 0


n_simul <-  10  # in the paper we did 1000 unless otherwise noted.
nrepet <- 39
nominal_level <-  0.05
(threshold <- (nominal_level + 2*sqrt(nominal_level*(1-nominal_level)/n_simul)))

# Gaussian response model settings
mu_max <- 10; tolerance = 2
rho_T = 0; rho_E = 1 # Eqn 1 and Eqn A.1/A.2

set.seed(1257)

pvals <- matrix(NA, nrow = n_simul, ncol = 7); 
colnames(pvals)=c("anova.traitglm", "glm_prow", "glm_pcol", "glm_pmax","4th_prow", "4th_pcol", "4th_pmax")

for (i in 1:n_simul){
    Tlist <- generate_Tdata(n_species, n_traits, cor_Tdat, b = rep(1,n_traits))
    Tdat <- Tlist$Tdat; T <-Tlist$T; c.true <-Tlist$coefficients
    E <- as.matrix(rnorm(n_communities))
    z <- rnorm(n_species); x <- rnorm(n_communities)
    z_star <- rho_T * T + sqrt(1 - rho_T^2)*z # rho_T = 0 == trait random; Eqn A.2 
    x_star <- rho_E * E + sqrt(1 - rho_E^2)*x # rho_E = 1 == env NOT random  Eqn A.2
    # thus: as rho_T = 0 and rho_E = 1
    # z_star = z # the trait governing the Gaussian model, but, alas, unobserved (latent)
    # x_star = E # the observed trait that is also the one governing the Gaussian model
    Y <- generate_community_Gaussian_response(tolerance = tolerance, E = x_star, T = z_star, mu_max = mu_max)
    obj <- make_obj_for_traitenv(Y,E,Tdat)
    res_anova.traitglm <- anova_traitglm(obj, nrepet = nrepet)
    res_glm_pmax <- pmax_PermutationTest(obj, trait_env_glm_LR, nrepet = nrepet)
    res_4th_pmax <- pmax_PermutationTest(obj, fourthcorner, nrepet = nrepet, withglm = FALSE)
    pvals[i,]= c(res_anova.traitglm["pval"], res_glm_pmax[-1],res_4th_pmax[-1])
}
#warnings() # glm.fit: fitted rates numerically 0 occurred
rejection_rate_q = colMeans(1*(pvals  <= nominal_level))
suspect.rej<- rejection_rate_q  > threshold 
print(rejection_rate_q)
# # anova.traitglm       glm_prow       glm_pcol       glm_pmax       4th_prow 
# # 0.9            0.7            0.0            0.0            0.6 
# # 4th_pcol       4th_pmax 
# # 0.1            0.1 

print(suspect.rej)
# # anova.traitglm       glm_prow       glm_pcol       glm_pmax       4th_prow 
# # TRUE           TRUE          FALSE          FALSE           TRUE 
# # 4th_pcol       4th_pmax 
# # FALSE          FALSE 
# # Conclusion 1: anova.traitglm gives highly elevated type I error (detectable even with n_simul = 10!)
# # Conclusion 2: glm_pmax gives controls the type I error rate
# # Conclusion 3: 4th_pmax gives controls the type I error rate
# # Remark: Inspection of pvals shows that in this many traits case, 
# # glm and 4th (actually the RLQ intertia) differ on a per data set basis;
# # this is not only due different in the permutation, but is generally expected, 
# # particularly so when cor_Tdat!=0 or the linear combination b does not simply specify a sum 

# Section 1b Gaussian response simulations with one random trait --------------------------------------------------

pvals <- matrix(NA, nrow = n_simul, ncol = 7); 
colnames(pvals)=c("anova.traitglm", "glm_prow", "glm_pcol", "glm_pmax","4th_prow", "4th_pcol", "4th_pmax")

for (i in 1:n_simul){
  T <- as.matrix(rnorm(n_species))
  E <- as.matrix(rnorm(n_communities))
  z <- rnorm(n_species);  x <- rnorm(n_communities)
  z_star <- rho_T * T + sqrt(1 - rho_T^2)*z # rho_T = 0 == trait random; Eqn A.2 
  x_star <- rho_E * E + sqrt(1 - rho_E^2)*x # rho_E = 1 == env NOT random  Eqn A.2
  # thus: if rho_T = 0 and rho_E = 1
  # z_star = z # the trait governing the Gaussian model, but, alas, unobserved (latent)
  # x_star = E # the observed trait that is also the one governing the Gaussian model
  Y <- generate_community_Gaussian_response(tolerance = tolerance, E = x_star, T = z_star, mu_max = mu_max)
  obj <- make_obj_for_traitenv(Y,E,T) 
  res_anova.traitglm <- anova_traitglm(obj, nrepet = nrepet)
# # in the next three lines, independent row and columns permutations are used for the two tests
#   res_glm_pmax <- pmax_PermutationTest(obj, trait_env_glm_LR, nrepet = nrepet)
#   res_4th_pmax <- pmax_PermutationTest(obj, fourthcorner, nrepet = nrepet)
#   pvals[i,]= c(res_anova.traitglm["pval"], res_glm_pmax[-1],res_4th_pmax[-1])
# # in the next line below, the same row and columns permutations are used for the two tests
# # so as to show that they give about the same result if n_env=n_traits=1
  glm_4th <- pmax_PermutationTest(obj, test_statistics_TE, nrepet = nrepet)
  pvals[i,]=c(res_anova.traitglm["pval"],glm_4th["glm",-1],glm_4th["fourth",-1])
}
#warnings() # glm.fit: fitted rates numerically 0 occurred
rejection_rate_1 = colMeans(1*(pvals  <= nominal_level))
suspect.rej<- rejection_rate_1  > threshold 
print(rejection_rate_1)
# anova.traitglm       glm_prow       glm_pcol       glm_pmax       4th_prow 
# 0.3            0.3            0.0            0.0            0.3 
# 4th_pcol       4th_pmax 
# 0.0            0.0 
print(suspect.rej)
# # anova.traitglm       glm_prow       glm_pcol       glm_pmax       4th_prow 
# # TRUE           TRUE          FALSE          FALSE           TRUE 
# # 4th_pcol       4th_pmax 
# # FALSE          FALSE 
# # Conclusion 1: anova.traitglm gives elevated type I error (detectable even with n_simul = 10!)
# # Conclusion 2: glm_pmax gives controls the type I error rate
# # Conclusion 3: 4th_pmax gives controls the type I error rate
# # Remark: Inspection of pvals shows that in this one-one case, 
# # glm and 4th do hardly differ on a per data set basis
pvals[, 1+(1:3)]- pvals[,4+(1:3)]
# # summary:
id = apply(abs(pvals[, 1+(1:3)]- pvals[,4+(1:3)]) > 0, 1, sum) 
sum(id);  # unequal in one data set
which(id >0) # namely the seven-th data set, if which(id >0) = 7 
pvals[id,]

rm(z, z_star, x_star, Y, E, T)

# Section 2a log-linear simulation model b_tx = 0 ---------------------------------------------

mu0 = 30
# effect sizes in log-linear model (b1,b2,b2 = b_te, b_ze, b_tx)
# with x and z random nuisance variables.
b_te <- 0   # the null model, without t-e interaction
b_ze <- 0.8
b_tx <- 0 
b_zx <- 0 
b_zx_star <- 0.2
sigma_epsilon_ij <- 0.2 # min(0.1, b_te + b_ze + b_tx)   # unstructured interaction


ef.main <- 0.05  # runif(1,min=0.1, max = ef.main)
ef.main.sq <-  -0.1
ef.main.ran.rows <- 0.1
ef.main.ran.columns <- 0.1

if (ef.main.sq < 0){ 
  opt <-  -ef.main/(2*ef.main.sq)
  tol <- 1/sqrt(-2*ef.main.sq)
} else {opt=NA;tol= NA}
print(opt); print(tol)

pvals <- matrix(NA, nrow = n_simul, ncol = 7); 
colnames(pvals)=c("anova.traitglm", "glm_prow", "glm_pcol", "glm_pmax","4th_prow", "4th_pcol", "4th_pmax")

for (i in 1:n_simul){
  T <- as.matrix(rnorm(n_species))
  E <- as.matrix(rnorm(n_communities))
  Y <- generate_RC_community(E= E, T= T, coefs=c(b_te, b_ze, b_tx, b_zx, b_zx_star), sigma_epsilon_ij  = sigma_epsilon_ij, mu0=mu0,
                                 coefsE=c(ef.main,ef.main.sq, ef.main.ran.rows), 
                                 coefsT=c(ef.main,ef.main.sq, ef.main.ran.columns))
  obj <- make_obj_for_traitenv(Y,E,T) 
  res_anova.traitglm <- anova_traitglm(obj, nrepet = nrepet)
  glm_4th <- pmax_PermutationTest(obj, test_statistics_TE, nrepet = nrepet)
  pvals[i,]=c(res_anova.traitglm["pval"],glm_4th["glm",-1],glm_4th["fourth",-1])
}
#warnings() # glm.fit: fitted rates numerically 0 occurred
rejection_rate_llm = colMeans(1*(pvals  <= nominal_level)) #llm = log-linear model
suspect.rej<- rejection_rate_llm  > threshold 
print(rejection_rate_llm)
# anova.traitglm       glm_prow       glm_pcol       glm_pmax       4th_prow 
# 0.3            0.2            0.0            0.0            0.2 
# 4th_pcol       4th_pmax 
# 0.0            0.0  
print(suspect.rej)
# anova.traitglm       glm_prow       glm_pcol       glm_pmax       4th_prow 
#       TRUE           TRUE          FALSE          FALSE           TRUE 
# 4th_pcol       4th_pmax 
# FALSE          FALSE 
# Conclusion 1: anova.traitglm gives elevated type I error (detectable even with n_simul = 10!)
# Conclusion 2: glm_pmax gives controls the type I error rate
# Conclusion 3: 4th_pmax gives controls the type I error rate
# Remark: Inspection of pvals shows that in this one-one case, 
# glm and 4th do differ slightly on a per data set basis
pvals[, 1+(1:3)]- pvals[,4+(1:3)]
# # summary:
id = apply(abs(pvals[, 1+(1:3)]- pvals[,4+(1:3)]) > 0, 1, sum) 
sum(id);  # unequal in ... data sets
which(id >0) # namely the seven-th data set, if which(id >0) = 7 
#pvals[id,]


# Section 2b log-linear simulation model b_tx != 0 single trait---------------------------------------------

n_simul <- 1000
# simulations with anova.traitglm as it was already shown to be wrong in this kind of model
# and it takes far too long with 1000 simulations...
mu0 <- 30
# effect sizes in log-linear model (b1,b2,b2 = b_te, b_ze, b_tx)
# with x and z random nuisance variables.
b_te <- 0   # the null model, without t-e interaction
b_ze <- 0.6
b_tx <- 0.6 
b_zx <- 0 
b_zx_star <- 0.2
sigma_epsilon_ij <- 0.2 # min(0.1, b_te + b_ze + b_tx)   # unstructured interaction


ef.main <- 0.05  # runif(1,min=0.1, max = ef.main)
ef.main.sq <-  -0.1
ef.main.ran.rows <- 0.1
ef.main.ran.columns <- 0.1

if (ef.main.sq < 0){ 
  opt <-  -ef.main/(2*ef.main.sq)
  tol <- 1/sqrt(-2*ef.main.sq)
}
print(opt); print(tol)

pvals <- matrix(NA, nrow = n_simul, ncol = 6); 
colnames(pvals)=c( "glm_prow", "glm_pcol", "glm_pmax","4th_prow", "4th_pcol", "4th_pmax")

for (i in 1:n_simul){
  T <- as.matrix(rnorm(n_species))
  E <- as.matrix(rnorm(n_communities))
  Y <- generate_RC_community(E= E, T= T, coefs=c(b_te, b_ze, b_tx, b_zx, b_zx_star), sigma_epsilon_ij  = sigma_epsilon_ij, mu0=mu0,
                             coefsE=c(ef.main,ef.main.sq, ef.main.ran.rows), 
                             coefsT=c(ef.main,ef.main.sq, ef.main.ran.columns))
  obj <- make_obj_for_traitenv(Y,E,T) 
  glm_4th <- pmax_PermutationTest(obj, test_statistics_TE, nrepet = nrepet)
  pvals[i,]=c(glm_4th["glm",-1],glm_4th["fourth",-1])
}
#warnings() # glm.fit: fitted rates numerically 0 occurred
rejection_rate_llm = colMeans(1*(pvals  <= nominal_level)) #llm = log-linear model
(threshold <- (nominal_level + 2*sqrt(nominal_level*(1-nominal_level)/n_simul)))
suspect.rej<- rejection_rate_llm  > threshold 
# glm_prow glm_pcol glm_pmax 4th_prow 4th_pcol 4th_pmax 
# 0.159    0.091    0.061    0.161    0.091    0.059 
# pmax of both glm and fourth corner ~ 0.06 that is: 
# slightly inflated with this smaller number of sites and species, although not significantly so 
print(rejection_rate_llm)
print(suspect.rej)
# glm_prow glm_pcol glm_pmax 4th_prow 4th_pcol 4th_pmax 
# TRUE    FALSE    FALSE     TRUE    FALSE    FALSE 
#

# Section 2c log-linear simulation model b_tx != 0 10 random traits---------------------------------------------

mu0 <- 30
# effect sizes in log-linear model (b1,b2,b2 = b_te, b_ze, b_tx)
# with x and z random nuisance variables.
b_te <- 0   # the null model, without t-e interaction
b_ze <- 0.6
b_tx <- 0.6 
b_zx <- 0 
b_zx_star <- 0.2
sigma_epsilon_ij <- 0.2 # min(0.1, b_te + b_ze + b_tx)   # unstructured interaction


ef.main <- 0.05  # runif(1,min=0.1, max = ef.main)
ef.main.sq <-  -0.1
ef.main.ran.rows <- 0.1
ef.main.ran.columns <- 0.1

if (ef.main.sq < 0){ 
  opt <-  -ef.main/(2*ef.main.sq)
  tol <- 1/sqrt(-2*ef.main.sq)
}
print(opt); print(tol)

pvals <- matrix(NA, nrow = n_simul, ncol = 6); 
colnames(pvals)=c( "glm_prow", "glm_pcol", "glm_pmax","4th_prow", "4th_pcol", "4th_pmax")

for (i in 1:n_simul){
  Tlist <- generate_Tdata(n_species, n_traits, cor_Tdat, b = rep(1,n_traits))
  Tdat <- Tlist$Tdat; T <-Tlist$T; c.true <-Tlist$coefficients
  E <- as.matrix(rnorm(n_communities))
  Y <- generate_RC_community(E= E, T= T, coefs=c(b_te, b_ze, b_tx, b_zx, b_zx_star), sigma_epsilon_ij  = sigma_epsilon_ij, mu0=mu0,
                             coefsE=c(ef.main,ef.main.sq, ef.main.ran.rows), 
                             coefsT=c(ef.main,ef.main.sq, ef.main.ran.columns))
  obj <- make_obj_for_traitenv(Y,E,Tdat) 
  glm_4th <- pmax_PermutationTest(obj, test_statistics_TE, nrepet = nrepet)
  pvals[i,]=c(glm_4th["glm",-1],glm_4th["fourth",-1])
}
#warnings() # glm.fit: fitted rates numerically 0 occurred
rejection_rate_llm = colMeans(1*(pvals  <= nominal_level)) #llm = log-linear model
(threshold <- (nominal_level + 2*sqrt(nominal_level*(1-nominal_level)/n_simul)))
suspect.rej<- rejection_rate_llm  > threshold 
# glm_prow glm_pcol glm_pmax 4th_prow 4th_pcol 4th_pmax 
# 0.769    0.028    0.020    0.531    0.055    0.046 
# pmax of both glm and fourth corner <0.05 that is: not inflated in this multi-trait setting
print(rejection_rate_llm)
print(suspect.rej)

